# TODO: Remove this in a near future

# I don't remove this directly because some of the structure
# and logic applied in these classes are interesting but due
# to some issues with the inheritance I decided to simplify
# the code, but I need to refactor and improve it